$! Extract table and domain information from an Rdb Database.
$! Write the information in RTF format for input to a word processor.
$!
$! B. Z. Lederman	06-Nov-1991
$!			12-Nov-1991	Add domains, look at comments
$!			18-Nov-1991	Add cross reference
$!			20-Nov-1991	Switch to program for cross reference
$!			21-Nov-1991	Use program for SQL command files,
$!					inital output file, table list.
$!			25-Nov-1991	Extract comments on tables,
$!					and indexes.
$!			21-Feb-1992	Take spaces out of comments (?)
$!
$ IF F$MODE () .EQS. "BATCH" THEN SET PROCESS /NAME = "Rdb to RTF"
$!
$ ON ERROR        THEN $ GOTO exitout
$ ON CONTROL_Y    THEN $ GOTO exitout
$ SET NOON
$!
$ IF F$TYPE (SQL$) .EQS. "" THEN $ SQL$ :== $SYS$SYSTEM:SQL$
$!
$ FALSE  = 0
$ TRUE   = -1
$ num_tab = 0
$ wanted      = FALSE
$ source_mode = FALSE
$ comment     = FALSE
$ index       = FALSE
$ table_name = "Dummy Table"
$!
$ IF P1 .EQS. ""			! if no input file specified
$ THEN
$    P1 = F$TRNLNM ("SQL$DATABASE")	! look for a default
$    IF P1 .EQS. ""			! no default
$    THEN
$	WRITE SYS$OUTPUT "Can't do anything without a database"
$	EXIT
$    ENDIF
$ ENDIF
$!
$! Parse the filespec to remove the type and replace it with ".RDB"
$!
$ file_name = F$PARSE (P1,,,"NAME")
$ file_dir  = F$PARSE (P1,,,"DIRECTORY")
$ rdb_file = file_dir + file_name + ".RDB"
$!
$ this_file = F$SEARCH (rdb_file, 1)
$ IF this_file .NES. ""  THEN $ GOTO got_file
$ WRITE SYS$OUTPUT "The file ''rdb_file doesn't exist"
$ EXIT
$!
$ got_file:
$!
$ DEFINE /NOLOG SQL$DATABASE 'this_file'
$!
$ IF F$SEARCH ("rdb_database.rtf") .NES. "" THEN -
  DELETE /NOLOG rdb_database.rtf;*
$ IF F$SEARCH ("rdb_tables.temp2") .NES. "" THEN -
  DELETE /NOLOG rdb_tables.temp2;*
$ IF F$SEARCH ("rdb_tables.temp4") .NES. "" THEN -
  DELETE /NOLOG rdb_tables.temp4;*
$ IF F$SEARCH ("rdb_tables_temp.rtf") .NES. "" THEN -
  DELETE /NOLOG rdb_tables_temp.rtf;*
$ IF F$SEARCH ("rdb_cross_temp.rtf") .NES. "" THEN -
  DELETE /NOLOG rdb_cross_temp.rtf;*
$ IF F$SEARCH ("rdb_fetch_tables_temp.sql") .NES. "" THEN -
  DELETE /NOLOG rdb_fetch_tables_temp.sql;*
$ IF F$SEARCH ("rdb_fetch_domains_temp.sql") .NES. "" THEN -
  DELETE /NOLOG rdb_fetch_domains_temp.sql;*
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 1"
$ ELSE
$    WRITE SYS$OUTPUT "Get list of tables, domains, and cross reference."
$ ENDIF
$!
$ RUN new_util:cross_reference
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 2"
$ ELSE
$    WRITE SYS$OUTPUT "Get domain descriptions from SQL."
$ ENDIF
$!
$ DEFINE SYS$OUTPUT NL:
$!
$ SQL$ @rdb_fetch_domains_temp.sql
$!
$ DEASSIGN SYS$OUTPUT
$!
$! Check exit status to see if there was a database access error?
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 3"
$ ELSE
$    WRITE SYS$OUTPUT "Append domain descriptions."
$ ENDIF
$!
$ OPEN /APPEND /ERROR = nofile outfile rdb_database.rtf
$ OPEN /READ   /ERROR = nofile tmpfile rdb_tables.temp4
$!
$ readloop3:
$!
$ READ /END = endfile3 tmpfile instring
$!
$ col1   = F$EXTRACT (0, 1, instring)
$!
$ IF col1 .EQS. "	"			! if line starts with tab
$ THEN						! should be multi-line
$!						! comment, compress it
$    WRITE outfile F$EDIT (instring, "COMPRESS, TRIM")
$    GOTO readloop3
$ ENDIF
$!
$ tmpstring = F$EDIT (instring, "COMPRESS, TRIM, UPCASE")
$!
$ first  = F$ELEMENT (0, " ", tmpstring)
$ second = F$ELEMENT (1, " ", tmpstring)
$!
$ IF first .EQS. "SHOW" .AND. second .EQS. "DOMAIN" THEN $ GOTO readloop3
$!
$ IF first .EQS. "EXIT" THEN $ GOTO endfile3
$!
$! The line with the field name must be formatted differently than
$! the comment and missing value lines. None of the field name lines
$! I've seen have a colon (":") in them, so that's what I'll use
$! to tell one line from another.
$!
$ IF F$LOCATE (":", tmpstring) .EQ. F$LENGTH (tmpstring)	! no colon
$ THEN							! must be field
$    field = F$ELEMENT (0, " ", tmpstring)
$    rest = tmpstring - field
$    WRITE outfile "\par\par"
$    WRITE outfile "{\b " + field + "}\tab" + rest + "\par"
$!!    WRITE SYS$OUTPUT field
$    GOTO readloop3
$ ENDIF
$!
$! Should probably parse here for tabs and replace with spaces or "\tab",
$! though hard tab characters are acceptable in RTF.  May have
$! to account for multi-line comments, however.
$!
$!! WRITE outfile instring + "\par"
$ WRITE outfile instring		! try no \par at end of comment
$!
$ GOTO readloop3
$!
$ endfile3:
$!
$ CLOSE tmpfile
$ CLOSE outfile
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 4"
$ ELSE
$    WRITE SYS$OUTPUT "Append list of tables."
$ ENDIF
$!
$ APPEND /NOLOG rdb_tables_temp.rtf rdb_database.rtf
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 5"
$ ELSE
$    WRITE SYS$OUTPUT "Get table descriptions from SQL."
$ ENDIF
$!
$ DEFINE SYS$OUTPUT NL:
$!
$ SQL$ @rdb_fetch_tables_temp.sql
$!
$ DEASSIGN SYS$OUTPUT
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 6"
$ ELSE
$    WRITE SYS$OUTPUT "Process table descriptions."
$ ENDIF
$!
$ OPEN /APPEND /ERROR = nofile outfile rdb_database.rtf
$ OPEN /READ   /ERROR = nofile tmpfile rdb_tables.temp2
$!
$ readloop4:
$!
$ READ /END = endfile4 tmpfile instring
$!
$ IF F$LENGTH (instring) .LE. 2 THEN GOTO readloop4	! skip if too short
$!
$ IF instring .EQS. "EXIT" THEN $ GOTO endfile4
$!
$ select = F$EXTRACT (0, 10, instring)
$ col1   = F$EXTRACT ( 0, 1, instring)
$!
$ IF select .EQS. "Table cons"
$ THEN
$    wanted      = FALSE
$    source_mode = FALSE
$    GOTO readloop4
$ ENDIF
$!
$ IF select .EQS. "Comment on"		! start capturing comments
$ THEN
$    comment = TRUE
$    GOTO readloop4
$ ENDIF
$!
$ IF comment
$ THEN
$    IF select .EQS. "Columns fo"	! stop capturing commetns
$    THEN
$	comment = FALSE
$	WRITE outfile "\par"
$	GOTO readloop4
$    ENDIF
$    WRITE outfile "\par " + instring
$    GOTO readloop4
$ ENDIF
$!
$ IF select .EQS. "Indexes on"		! start capturing index information
$ THEN
$    index = TRUE
$    WRITE outfile "\par\par{\b\ul Indexes}\par"
$    GOTO readloop4
$ ENDIF
$!
$ IF index
$ THEN
$    IF select .EQS. "Storage Ma" .OR. select .EQS. "No indexes"
$    THEN
$	index = FALSE			! stop capturing index information
$	GOTO readloop4
$    ENDIF
$!
$    tmpstring = F$EDIT (instring, "COMPRESS, TRIM")
$!
$    IF col1 .NES. "	"			! first line in index
$    THEN
$	domain = F$ELEMENT (0, " ", tmpstring)
$	tmpstring = tmpstring - domain
$	tmpstring = "{\b " + domain + "}\tab" + tmpstring
$    ELSE					! other lines in group
$	tmpstring = "\tab " + tmpstring
$    ENDIF
$!
$    WRITE outfile "\par" + tmpstring
$    GOTO readloop4
$ ENDIF
$!
$ IF select .EQS. "SHOW TABLE"
$ THEN
$    table_name = F$ELEMENT (2, " ", instring)
$    wanted      = FALSE
$    source_mode = FALSE
$!!    WRITE SYS$OUTPUT table_name		! so we can see progress
$    WRITE outfile "\page\par\pard \tx2800\tx4600"
$!
$! May consider putting the database file name back in here.
$!
$    WRITE outfile "{\b Columns for " + table_name + "}\par"
$    GOTO readloop4
$ ENDIF
$!
$ IF select .EQS. "----------"
$ THEN
$    wanted = TRUE
$!    WRITE outfile "\par{\ul Column Name}{\ul \tab }{\ul Data Type}{\ul \tab}{\ul Domain}\par"
$    WRITE outfile "\par{\ul Column Name\tab Data Type\tab Domain}\par"
$    GOTO readloop4
$ ENDIF
$!
$ IF .NOT. wanted THEN $ GOTO readloop4		! not ready for output yet
$!
$! Have to extract field names from everything else.  Fields start
$! in column 1 and have a blank space (or should have) in columns 31
$! and 32.
$!
$ col31 = F$EXTRACT (30, 1, instring)
$!
$ IF col1 .NES. " " .AND. col31 .EQS. " "
$ THEN
$    tmpstring = F$EDIT (F$ELEMENT (0, "	", instring), "TRIM")
$    domain = F$EDIT (F$ELEMENT (1, "	", instring), "COLLAPSE, TRIM")
$    IF domain .EQS. "" THEN domain = F$EDIT (F$ELEMENT (2, "	", instring), "COLLAPSE, TRIM")
$    column = F$ELEMENT (0, " ", tmpstring)
$    length = F$LENGTH (tmpstring) - 32
$    data_t = F$EXTRACT (32, length, tmpstring)
$    WRITE outfile "\par"
$    WRITE outfile "{\b " + column + "}\tab " + data_t + "\tab " + domain
$    GOTO readloop4
$ ENDIF
$!
$! Still have to look for the start of a source line.
$!
$ select = F$EXTRACT (0, 8, select)
$ IF select .EQS. " Source:"
$ THEN
$    WRITE outfile "\par\par { Source:}"
$!
$! Strip the <CR><LF> out of the line
$!
$    WRITE outfile "\par" + F$EXTRACT (10, F$LENGTH(instring) - 10, instring)
$    source_mode = TRUE
$    GOTO readloop4
$ ENDIF
$!
$! Have to look for Missing Values, which are not part of comments.
$! Start them on their own line. Also look for View Source lines.
$!
$ IF select .EQS. " Missing" .OR. select .EQS. " Comment" .OR. source_mode
$ THEN
$    WRITE outfile "\par" + instring
$    GOTO readloop4
$ ENDIF
$!
$! Parse tabs out of comments and similar fields.
$!
$ instring = F$EDIT (instring, "COMPRESS, TRIM")
$ WRITE outfile instring
$ GOTO readloop4
$!
$ endfile4:
$!
$ CLOSE tmpfile
$ CLOSE outfile
$!
$ IF F$MODE () .EQS. "BATCH"
$ THEN
$    SET PROCESS /NAME = "Rdb to RTF 7"
$ ELSE
$    WRITE SYS$OUTPUT "Append cross-reference of domains to columns."
$ ENDIF
$!
$ APPEND /NOLOG rdb_cross_temp.rtf rdb_database.rtf
$!
$ exitout:
$!
$ SET NOON
$!
$ IF P2 .NES. ""			! if an output file was specified
$ THEN
$    RENAME /NOLOG rdb_database.rtf 'P2'
$ ENDIF
$!
$ DELETE /NOLOG rdb_tables.temp2;*
$ DELETE /NOLOG rdb_tables.temp4;*
$ DELETE /NOLOG rdb_tables_temp.rtf;*
$ DELETE /NOLOG rdb_cross_temp.rtf;*
$ DELETE /NOLOG rdb_fetch_tables_temp.sql;*
$ DELETE /NOLOG rdb_fetch_domains_temp.sql;*
$!
$ nofile:
$!
$ EXIT $STATUS
